@@ -8,7 +8,11 @@ import numpy as np
 import pandas as pd
 import re
 
-def add_date_features(df, date_vars, drop = True, time = False):
+def add_date_features(df, 
+                      date_vars, 
+                      drop = True, 
+                      time = False, 
+                      inplace = False):
     '''
     Adds basic date-based features based to the data frame.
 
@@ -18,6 +22,7 @@ def add_date_features(df, date_vars, drop = True, time = False):
     - date_var (str): name of the date feature
     - drop (bool): whether to drop the original date feature
     - time (bool): whether to include time-based features
+    - inplace (bool): whether to add features in place or return a modified data set
 
     --------------------
     Returns:
@@ -38,6 +43,9 @@ def add_date_features(df, date_vars, drop = True, time = False):
     df_new = add_date_features(df, date_vars = 'date_of_birth')
 
     '''
+    # store original data
+    if inplace == False:
+        df_original = df.copy()
     
     # store no. features
     n_feats = df.shape[1]
@@ -83,7 +91,10 @@ def add_date_features(df, date_vars, drop = True, time = False):
 
     # return results
     print('Added {} date-based features.'.format(df.shape[1] - n_feats + int(drop) * len(date_vars)))
-    return df
+    if inplace == False:
+        df_new = df.copy()
+        df     = df_original.copy()
+        return df_new
 
 
 
@@ -102,7 +113,8 @@ def add_text_features(df,
                       tf_idf_feats = 5, 
                       common_words = 0,
                       rare_words = 0,
-                      drop = True):
+                      drop = True,
+                      inplace = False):
     '''
     Adds basic text-based features including word count, character count and 
     TF-IDF based features to the data frame.
@@ -115,6 +127,7 @@ def add_text_features(df,
     - common_words (int): number of the most common words to remove for TF-IDF
     - rare_words (int): number of the most rare words to remove for TF-IDF
     - drop (bool): whether to drop the original textual features
+    - inplace (bool): whether to add features in place or return a modified data set
 
     --------------------
     Returns:
@@ -139,6 +152,10 @@ def add_text_features(df,
     df_new = add_text_features(df, text_vars = ['income', 'gender'])
     '''
 
+    # store original data
+    if inplace == False:
+        df_original = df.copy()
+
     # store no. features
     n_feats = df.shape[1]
 
@@ -189,7 +206,10 @@ def add_text_features(df,
         
     # return results
     print('Added {} text-based features.'.format(df.shape[1] - n_feats + int(drop) * len(text_vars)))
-    return df
+    if inplace == False:
+        df_new = df.copy()
+        df     = df_original.copy()
+        return df_new
 
 
 
@@ -339,7 +359,7 @@ def aggregate_data(df,
 
 import pandas as pd
 
-def encode_factors(df, method = 'label'):
+def encode_factors(df, method = 'label', inplace = False):
     '''
     Performs encoding of categorical features using label or dummy encoding.
 
@@ -347,6 +367,7 @@ def encode_factors(df, method = 'label'):
     Arguments:
     - df (pandas DF): pandas DF
     - method (str): encoding method ('label' or 'dummy')
+    - inplace (bool): whether to add features in place or return a modified data set
 
     --------------------
     Returns:
@@ -369,6 +390,9 @@ def encode_factors(df, method = 'label'):
     from dptools import encode_factors
     df_enc = encode_factors(df, method = 'label')
     '''
+    # store original data
+    if inplace == False:
+        df_original = df.copy()
     
     # label encoding
     if method == 'label':
@@ -380,4 +404,8 @@ def encode_factors(df, method = 'label'):
     if method == 'dummy':
         df = pd.get_dummies(df, drop_first = True)
 
-    return df
\ No newline at end of file
+    # return data
+    if inplace == False:
+        df_new = df.copy()
+        df     = df_original.copy()
+        return df_new
\ No newline at end of file
